/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;

/**
 * Java component implementation with
 * 1 service with interface Service1
 * 1 reference (1..1) with interface Service1
 * 
 * The implementation is
 * - COMPOSITE scope
 * - is marked @EagerInit to ensure that it is started as soon as its containing composite is started
 * - has an @Init method
 * - the @Init method invokes the reference 
 */
@Service(Service1.class)
@Scope("COMPOSITE")
@EagerInit
public class compositeEagerInitImpl implements Service1 {
	
	@Property
	public String serviceName = "service1";
	// Required = true -> multiplicity 1..1
	@Reference(required=true)
	public Service1 reference1 = null;
	
	private String initialResponse;

	public String operation1(String input) {
		return serviceName + " operation1 invoked" + initialResponse;
	} // end method operation1
	
	@Init
	public void initializer() {
		// Invoke the reference operation and store the result
		initialResponse = reference1.operation1("EagerInit called");
	} // end method initializer

} // end class compositeEagetInitImpl
