/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.RequestContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.annotation.Context;
import org.oasisopen.sca.annotation.Property;
import org.oasisopen.sca.annotation.Reference;
import org.oasisopen.sca.annotation.Service;
import org.oasisopen.sca.test.Service1;

/**
 * Java component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service3WithCallback which has a Callback
 * interface Service3Callback which is implemented by this class
 * reference gets called when service1 operation1 is invoked
 * 
 * 
 */
@Service(Service1.class)
public class service1CallbackContextImpl1 implements Service1, Service3Callback {

	private ComponentContext componentContext;
	
	@Property
	public String serviceName = "service1";
	// Reference with an implied multiplicity of (1..1) since required=true is the default
	@Reference
	public Service3WithCallback reference1 = null;
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		String result = reference1.operation3(input);
		return serviceName + " operation1 invoked" + " " + result;
	}
	
	public String callback1( String input ) {
		if( "callback".equals(input) ) return regularCallback( input );
		if( "check".equals(input) ) return checkCallback( input );
		return serviceName + " invalid invocation of callback operation callback1";
	} // end 
	
	private String regularCallback( String input ) {
		String result = "";
		String response = serviceName + " callback1 invoked";
		
		RequestContext theReqContext;
		try {
			theReqContext = componentContext.getRequestContext();
			if( theReqContext == null ) return response + " null RequestContext received from getRequestContext()";
		} catch (Exception e) {
			return response + " exception received from getRequestContext: " + e.toString();
		} // end try
		
		// Obtain the ServiceReference from the RequestContext
		ServiceReference<?> theServRef = theReqContext.getServiceReference();
		if( theServRef == null ) return response + " getServiceReference returned a null ServiceReference";
		
		// Get the business interface from the ServiceReference and check it is the right interface
		Class<?> theBusinessInterface = theServRef.getBusinessInterface();
		if( !(theBusinessInterface == Service3Callback.class) ) {
			return response + " getServiceReference returned incorrect business interface: " + theBusinessInterface.toString();
		} // end if
		
		// Get a service reference that can be invoked...
		Service3Callback referenceService = (Service3Callback) theServRef.getService();
		
		// Call a service operation on the service reference - it should actually call THIS same component
		result += "ServiceReference obtained: ";
		result += referenceService.callback1("check");

		return  response + " " + result;
	} // end regularCallback
	
	/**
	 * This method is used to check that the returned ServiceReference does indeed point to THIS
	 * service implementation
	 */
	private String checkCallback( String input ) {
		return serviceName + " " + input + " callback1 invoked";
	} // end checkCallback

}
