/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.Callback;
import org.oasisopen.sca.annotation.Property;
import org.oasisopen.sca.annotation.Service;

/**
 * Java component implementation which offers multiple bidirectional services (ie it has multiple callbacks)
 * 1 service with interface Service3WithCallback
 * - callback using interface Service3Callback
 *
 * 1 service with interface Service7WithCallback
 * - callback using interface Service7Callback
 * 
 * 1 service with interface Service1
 * - no callback
 *
 * 0 references
 * 0 properties
 * 
 * This implementation has both Fields and Setters for the injection of the Callback objects for both
 * services with callbacks.  When the operation of the callback services is invoked, the operation invokes
 * the callback twice - once using the Field callback object and once using the Setter callback object.
 * The operation also checks the status of the callback objects for the other (not invoked) service, which
 * should be null.
 */
@Service({Service1.class, Service3WithCallback.class, Service7WithCallback.class})
public class MultipleCallbacksImpl implements Service1, Service3WithCallback, Service7WithCallback {
	
	@Property
	public String serviceName = "service3";
	// Field for injection of callback for Service3
	@Callback
	public Service3Callback callback3;
	// Field for injection of callback for Service7
	@Callback
	public Service7Callback callback7;
	
	private Service3Callback setterCallback3 = null;
	private Service7Callback setterCallback7 = null;
	
	/**
	 * Service operation for Service1 which is not bidirectional
	 */
	public String operation1(String input) {
		String callbacks = "callbacks null";
		if( callback3 != null || setterCallback3 != null ) callbacks = "callbacks also set";
		if( callback7 != null || setterCallback7 != null ) callbacks = "callbacks also set";
		return serviceName + " operation1 invoked " + callbacks;
	}

	/**
	 * Service operation for Service3 which invokes an operation on the callback Service3Callback
	 */
	public String operation3(String input) {
		String callbackResult = callback3.callback1("callback");
		String callbackResult2 = setterCallback3.callback1("callback2");
		String otherCallbacks = "other callbacks null";
		if( callback7 != null || setterCallback7 != null ) otherCallbacks = "other callbacks also set";

		return serviceName + " operation3 invoked" + " " + callbackResult + " " + callbackResult2 + " " + otherCallbacks;
	}

	/**
	 * Service operation for Service7 which invokes an operation on the callback Service7Callback
	 */
	public String operation7(String input) {
		String callbackResult = callback7.service7Callback1("callback");
		String callbackResult2 = setterCallback7.service7Callback1("callback2");
		String otherCallbacks = "other callbacks null";
		if( callback3 != null || setterCallback3 != null ) otherCallbacks = "other callbacks also set";

		return serviceName + " operation7 invoked" + " " + callbackResult + " " + callbackResult2 + " " + otherCallbacks;
	}

	/**
	 * Setter method for the callback for Service3
	 * @param theCallback
	 */
	@Callback
	public void setService3Callback( Service3Callback theCallback ) {
		setterCallback3 = theCallback;
	}
	
	/**
	 * Setter method for the callback for Service7
	 * @param theCallback
	 */
	@Callback
	public void setService7Callback( Service7Callback theCallback ) {
		setterCallback7 = theCallback;
	}
} // end class
