/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.oasisopen.sca.ResponseDispatch;
import org.oasisopen.sca.annotation.*;


/**
 * Java component implementation
 * 1 service with interface Service1AsyncServer
 * 0 references
 *
 * Async server implementation of the Service1 service
 *
 */
@Service(Service1AsyncServer.class)
public class Service1AsyncServerImpl implements Service1AsyncServer {
	
	private volatile ResponseDispatch<String> responseHandler = null;
	
	@Property(required=true)
	public volatile String serviceName = "service1";
	
	/*
	public String operation1(String input) {
		return serviceName + " operation1 invoked";
	}
	*/

	public void operation1Async(String input, ResponseDispatch<String> handler) {
		// Store the ResponseDispatch object
		responseHandler = handler;
		
		// Now kick off processing on a separate thread that will dispatch the response some time after this
		// initial method returns...
		runResponseThread( input );
		
		// return
		return;
	} // end method operation1Async
	
	/**
	 * Method used to run a separate thread, to invoke the ResponseDispatch
	 */
	private void runResponseThread( String input ) {
		
		int invocationCount = 2;	// # of threads to use
		long maxWaitTime = 5000;	// Max wait time for completion = 5sec
		
		// Run the tests using a ThreadPoolExecutor
		ThreadPoolExecutor theExecutor = new ThreadPoolExecutor( 	invocationCount, invocationCount,
													                maxWaitTime, TimeUnit.MILLISECONDS,
													                new ArrayBlockingQueue<Runnable>( invocationCount ) );
		

		
		// Perform the invocations on separate thread...
		theExecutor.execute( new separateThreadInvoker( input ) );

	} // end method runResponseThread

	/**
	 * An inner class which acts as a runnable task for invoking APIs on threads that are not processing
	 * either a service operation or a callback operation
	 */
	private class separateThreadInvoker implements Runnable {
		
		private long pauseTime = 1000;		// Pause interval to allow initiating thread to complete
		private String input;				// Input parameter
				
		public separateThreadInvoker( String input ) {
			super();
			this.input = input;
		} // end constructor

		public void run() {

			// Wait for a short time to ensure that the invoking thread has time to return
			try {
				Thread.sleep(pauseTime);
			} catch (InterruptedException e) {
				// Nothing to do here...
			}
			
			if( "exception".equals(input) ) {
				// Invoke the response dispatch object to return a an exception
				responseHandler.sendFault( new BusinessFault1(serviceName + " operation1 invoked asynchronously"));

			} else {
				// Invoke the response dispatch object to return a response
				responseHandler.sendResponse( serviceName + " operation1 invoked asynchronously");
			} // end if 
		} // end method run
		
	} // end class separateThreadInvoker

}
