/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 0 references 
 * 
 * This implementation performs invocations of the createSelfReference( Class<B> ) method of the ComponentContext API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1Superset.class)
public class Service1CCcreateSelfReferenceAImpl implements Service1Superset {
	
	private ComponentContext componentContext;
	private String responseAccumulator = "";
	
	@Property
	public String serviceName = "service1";

	public Service1CCcreateSelfReferenceAImpl() {
		super();
	}
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked createSelfReference";
		
		if( componentContext == null ) return responseAccumulator + " injected componentContext null";
		
		// JCA-TA-9026 - IllegalArgumentException thrown if this component does not have a service with the businessInterface specified
		try {
		    componentContext.createSelfReference(Service2.class);
		    responseAccumulator += " did not get IllegalArgumentException for invalid businessInterface";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid businessInterface";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid businessInterface: " + e.toString();
		} // end try	
		
		// JCA-TA-9025 - returns a valid ServiceReference for one of the services offered by this component
		try {
		    ServiceReference<Service1Superset> serviceRef = componentContext.createSelfReference(Service1Superset.class);
		    if( serviceRef == null ) {
			    responseAccumulator += " unexpected null ServiceReference";
		    } else {
		    	responseAccumulator += " expected non-null ServiceReference";
		    	// Actually invoke the target service to verify that the returned ServiceReference is indeed a proxy for that service
		    	Service1Superset targetService = serviceRef.getService();
		    	responseAccumulator += " " + targetService.operation2( "dynamic" );
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received: " + e.toString();
		} // end try		

		return responseAccumulator;
	} // end operation1

	public String operation2(String input) {
		return serviceName + " operation2 invoked";
	}

} // end class
