/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.ResponseDispatch;
import org.oasisopen.sca.annotation.AsyncFault;
import org.oasisopen.sca.annotation.AsyncInvocation;
import org.oasisopen.sca.annotation.Remotable;

/**
 * Service1 service interface with with a series of declared business faults on operation1
 * - Asynchronous server version
 */
@AsyncInvocation
@Remotable
public interface Service1WithFaultsAsyncServer  {
	
	/**
	 * Synchronous method for invoking testcase service
	 * @param input - input parameter as a String
	 * @return - output data as a String
	 * @throws - BusinessFault1, BusinessFault2 - two different exceptions
	 * Listed here for documentation purposes - this is the operation that the async server operation maps to
	 */
	// public String operation1( String input ) throws BusinessFault1, BusinessFault2;
	
	/**
	 * Async server version of the synchronous operation1 method
	 * @param input - input parameter as a String
	 * @param handler - the ResponseDispatch<String> handler used to send the response message (a String in this case)
	 * @AsyncFault annotation lists the business exceptions which may be sent back from this method
	 */
	@AsyncFault({BusinessFault1.class, BusinessFault2.class})
	public void operation1Async( String input, ResponseDispatch<String> handler );
	
} // end interface Service1WithFaultsAsyncServer
