/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;
import org.oasisopen.sca.test.MultipleService;

/**
 * Java component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface MultipleService
 * The reference gets called multiple times in parallel when service1 operation1 is invoked
 *
 */
@Service(Service1.class)
public class multipleServiceClientImpl implements Service1 {
	
	@Property
	public String serviceName = "service1";
	// Number of invocations of all the operations of the reference reference1 - default = 50
	@Property
	public int invocationCount = 50;		
	// Reference with an implied multiplicity of (1..1) since required=true is the default
	@Reference
	public MultipleService reference1 = null;
	
	public String operation1(String input) {
		int accumulator = 0;
		// Loop over all of the operations of reference1, calling each in sequence, for the number
		// of iterations specified in the invocationCount SCA Property
		// Accumulate the results - 0 implies that no target Java instance was ever invoked on more 
		// than one method > 0 implies that at least one instance was invoked more than once
		String parameter = "input";
		for( int i = 0; i < invocationCount; i++ ) {
			accumulator += reference1.operation1(parameter);
			accumulator += reference1.operation2(parameter);
			accumulator += reference1.operation3(parameter);
			accumulator += reference1.operation4(parameter);
			accumulator += reference1.operation5(parameter);
			accumulator += reference1.operation6(parameter);
			accumulator += reference1.operation7(parameter);
			accumulator += reference1.operation8(parameter);
			accumulator += reference1.operation9(parameter);
			accumulator += reference1.operation10(parameter);
		} // end for
		
		// Check whether any of the target service invocations experienced
		// multiple invocations of any given instance of the target service implementation
		// - if this ever happens then the accumulator will be non-zero
		if( accumulator == 0 ) {
			return serviceName + " multiple service invocation successful";
		} else {
			return serviceName + " multiple service invocation failed - number of multiple invocations of an instance = "
			                   + (accumulator + 1);
		} // end if
	} // end method operation1	

} // end class multipleServiceClient
