/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 1 reference (1..n) with interface Service1
 * all configured wires get called when service1 operation1 is invoked
 * 
 * This implementation invokes the ComponentContext getService() API for the reference
 * and expects to receive an IllegalArgumentException from that method
 */
@Service(Service1.class)
public class service1ContextImpl1 implements Service1 {
	
	private ComponentContext componentContext;
	
	@Property
	public String serviceName = "service1";
	// Required = true + an array -> multiplicity 1..n
	@Reference(required=true)
	public Service1[] reference1 = null;
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		String result = "";
		String response = serviceName + " operation1 invoked";
		
		try {
			componentContext.getService(Service1.class, "reference1");
		} catch (IllegalArgumentException ia) {
			return response + " IllegalArgumentException received";
		} catch (Exception e) {
			return response + " other exception received from getService: " + e.toString();
		} // end try

		// Deal with the case where the reference has no targets supplied...
		// Should this throw an exception?
		if( reference1.length == 0 ) {
			// throw new InvalidSCAConfigurationException("1..n multiplicity reference has no targets");
			return serviceName + " 1..n multiplicity reference has no targets";
		} // end if
		
		// Call each of the references in the array, concatenating the results
		for( int i=0 ; i < reference1.length; i++ ) {
			result = result.concat( reference1[i].operation1(input) );
		} // end for
		return  response + " " + result;
	}

}
