/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.ServiceOneWay;


/**
 * Java component implementation
 * 1 service with interface ServiceOneWay
 * 0 references
 *
 * The implementation is COMPOSITE scope and is stateful
 * - it remembers whether its OneWay method inonly1 was called
 * - it reports this when its request/Response method is called
 */
@Service(ServiceOneWay.class)
@Scope("COMPOSITE")
public class serviceOneWayImpl implements ServiceOneWay {
	
	String status = "OneWay method inonly not called";
	
	CountDownLatch theLatch = new CountDownLatch( 1 );
	
	@Property(required=true)
	public String serviceName = "service1";
	
	/*
	 * A problem in the handling of the OneWay and Req/Resp methods of this service
	 * is that the OneWay method is dispatched asynchronously from the client while
	 * the Req/Resp is dispatched synchronously.  This can mean that the client program
	 * invokes the OneWay and then the Req/Resp but that those invocations arrive at this
	 * service implementation in the opposite order.  To deal with this, a latch is used
	 * which causes the Req/Resp operation to wait a while for the OneWay method invocation
	 * to complete.
	 */
	public String operation1(String input) {
		try {
			theLatch.await( 1, TimeUnit.SECONDS );
		} catch (InterruptedException e) {
			// Do nothing
		}
		return serviceName + " operation1 invoked " + status;
	}

	@OneWay
	public void inonly1(String input) {
		status = "OneWay method inonly previously called with " + input;
		theLatch.countDown();
	} // end inonly1

} // end class serviceOneWayImpl
